# The STARK field
# https://docs.starknet.io/documentation/architecture_and_concepts/Cryptography/p-value/
PRIME = 2**251 + 17 * 2**192 + 1

PARACLEAR_DECIMALS = 8
USDC_DECIMALS = 6

BUY_SIDE = 1
SELL_SIDE = 2

WS_READ_TIMEOUT = 5
